import sys

from django.apps import AppConfig as DjangoAppConfig


class AppConfig(DjangoAppConfig):
    name = "edc_lab_dashboard"
    verbose_name = "Edc Lab Dashboard"
    include_in_administration_section = False
    admin_site_name = "edc_lab_admin"

    def ready(self):
        sys.stdout.write(f"Loading {self.verbose_name} ...\n")
        sys.stdout.write(f" Done loading {self.verbose_name}.\n")



from edc_lab.models import BoxItem

from ...view_mixins import BoxViewMixin
from .base_listboard_view import BaseListboardView


class BaseBoxItemListboardView(BoxViewMixin, BaseListboardView):
    navbar_selected_item = "pack"
    ordering = ("-position",)
    listboard_model = BoxItem
    listboard_view_permission_codename = "edc_lab_dashboard.view_lab_box_listboard"

    def get_queryset_filter_options(self, request, *args, **kwargs):
        return {"box": self.box}



from django.contrib.sites.models import Site
from django.test import TestCase
from django.urls.base import reverse
from edc_lab.models import (
    Aliquot,
    Box,
    BoxItem,
    BoxType,
    Consignee,
    Manifest,
    ManifestItem,
    Shipper,
)

from edc_lab_dashboard.model_wrappers import (
    BoxModelWrapper,
    ManageBoxItemModelWrapper,
    ManifestItemModelWrapper,
    ManifestModelWrapper,
)


class TestModelWrapper(TestCase):
    def setUp(self):
        self.box_type = BoxType.objects.create(name="9 x 9", across=9, down=9, total=81)
        self.box = Box.objects.create(box_identifier="12345678", box_type=self.box_type)
        self.box_item = BoxItem.objects.create(box=self.box, position=0)
        self.aliquot = Aliquot.objects.create(
            subject_identifier="ABCDEFG",
            count=1,
            is_primary=True,
            aliquot_type="Whole Blood",
            numeric_code="02",
            alpha_code="WB",
        )

    def test_box_model_wrapper(self):
        wrapper_cls = BoxModelWrapper
        box_type = BoxType.objects.create(across=9, down=9, total=81)
        obj = Box.objects.create(box_identifier="1234", box_type=box_type)
        wrapper = wrapper_cls(obj)
        self.assertEqual(
            wrapper.href,
            f"/admin/edc_lab/box/{obj.id}/change/?"
            f"next=edc_lab_dashboard:pack_listboard_url&",
        )
        self.assertEqual(
            reverse("edc_lab_dashboard:pack_listboard_url"),
            "/edc_lab_dashboard/listboard/pack/",
        )

    def test_manage_box_item_model_wrapper(self):
        wrapper_cls = ManageBoxItemModelWrapper
        # usually this will come from app_config
        box_type = BoxType.objects.create(across=9, down=9, total=81)
        box_identifier = "1234"
        box = Box.objects.create(box_identifier=box_identifier, box_type=box_type)
        obj = BoxItem.objects.create(box=box, position=0, identifier="1234")
        wrapper = wrapper_cls(obj)
        self.assertEqual(
            wrapper.href,
            f"/admin/edc_lab/boxitem/{obj.id}/change/?"
            f"next=edc_lab_dashboard:manage_box_listboard_url,"
            f"box_identifier,action_name&box_identifier={box_identifier}&"
            f"action_name=manage&",
        )

    def test_manifest_item_model_wrapper(self):
        wrapper_cls = ManifestItemModelWrapper
        manifest = Manifest.objects.create(
            site=Site.objects.get_current(),
            consignee=Consignee.objects.create(name="name"),
            shipper=Shipper.objects.create(name="name"),
        )
        obj = ManifestItem.objects.create(manifest=manifest)
        wrapper = wrapper_cls(obj)
        self.assertEqual(
            wrapper.href,
            f"/admin/edc_lab/manifestitem/{obj.id}/change/?"
            f"next=edc_lab_dashboard:manage_manifest_listboard_url&",
        )

    def test_manifest_model_wrapper(self):
        wrapper_cls = ManifestModelWrapper
        obj = Manifest.objects.create(
            site=Site.objects.get_current(),
            consignee=Consignee.objects.create(name="name"),
            shipper=Shipper.objects.create(name="name"),
        )
        wrapper = wrapper_cls(obj)
        self.assertEqual(
            wrapper.href,
            f"/admin/edc_lab/manifest/{obj.id}/change/?"
            f"next=edc_lab_dashboard:manifest_listboard_url&",
        )
        self.assertEqual(
            reverse("edc_lab_dashboard:manifest_listboard_url"),
            "/edc_lab_dashboard/listboard/manifest/",
        )



from edc_dashboard.url_names import url_names
from edc_lab.constants import SHIPPED
from edc_lab.models import Manifest
from edc_lab.reports import ManifestReport

from ...model_wrappers import ManifestModelWrapper
from ..listboard_filters import ManifestListboardViewFilters
from .base_listboard_view import BaseListboardView


class ManifestListboardView(BaseListboardView):
    navbar_selected_item = "manifest"

    form_action_url = "manifest_form_action_url"
    listboard_url = "manifest_listboard_url"
    listboard_template = "manifest_listboard_template"
    listboard_model = Manifest
    listboard_view_permission_codename = "edc_lab_dashboard.view_lab_manifest_listboard"
    listboard_view_only_my_permission_codename = None
    model_wrapper_cls = ManifestModelWrapper
    listboard_view_filters = ManifestListboardViewFilters()
    search_form_url = "manifest_listboard_url"
    print_manifest_url = "print_manifest_url"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update(
            new_manifest=ManifestModelWrapper(Manifest()),
            print_manifest_url_name=url_names.get(self.print_manifest_url),
            SHIPPED=SHIPPED,
        )
        return context

    def get(self, request, *args, **kwargs):
        if request.GET.get("pdf"):
            response = self.print_manifest()
            return response
        return super().get(request, *args, **kwargs)

    @property
    def manifest(self):
        return Manifest.objects.get(manifest_identifier=self.request.GET.get("pdf"))

    def print_manifest(self):
        manifest_report = ManifestReport(manifest=self.manifest, user=self.request.user)
        return manifest_report.render_to_response()
